from typing import List, Optional

from frostfs_sdk.models.enums.filter_operation import FilterOperation
from frostfs_sdk.models.dto.filter import Filter
from frostfs_sdk.protos.models.netmap import types_pb2 as types_pb2_netmap


class FilterMapper:
    @staticmethod
    def to_grpc_messages(filters: List[Filter]) -> List[types_pb2_netmap.Filter]:
        """
        Converts list of Filter DTOs to gRPC messages with nested conversion
        """
        if not filters:
            return []
        
        return [FilterMapper.to_grpc_message(f) for f in filters]

    @staticmethod
    def to_grpc_message(filter_dto: Filter) -> types_pb2_netmap.Filter:
        """
        Converts Filter DTO to gRPC message with nested filters
        """

        operation = types_pb2_netmap.Filter.Operation.Value(filter_dto.operation.value)        
        return types_pb2_netmap.Filter(
            name=filter_dto.name,
            key=filter_dto.key,
            op=operation,
            value=filter_dto.value,
            filters=FilterMapper.to_grpc_messages(filter_dto.filters)
        )

    @staticmethod
    def to_models(filters_grpc: List[types_pb2_netmap.Filter]) -> Optional[List[Filter]]:
        """
        Converts gRPC messages to Filter DTOs with nested conversion
        """
        if not filters_grpc:
            return None
        
        return [FilterMapper.to_model(f) for f in filters_grpc]

    @staticmethod
    def to_model(filter_grpc: types_pb2_netmap.Filter) -> Optional[Filter]:
        """
        Converts gRPC message to Filter DTO with nested filters
        """
        if not filter_grpc or filter_grpc.ByteSize() == 0:
            return None
        
        operation = FilterOperation(filter_grpc.op)
        
        return Filter(
            name=filter_grpc.name,
            key=filter_grpc.key,
            operation=operation,
            value=filter_grpc.value,
            filters=FilterMapper.to_models(filter_grpc.filters)
        )
